@@ -23,8 +23,8 @@ except ImportError:
 
 from u1db.errors import InvalidJSON, InvalidContent
 
-__version_info__ = (0, 1, 4)
-__version__ = '.'.join(map(str, __version_info__))
+__version_info__ = (13, 9)
+__version__ = '.'.join(map(lambda x: '%02d' % x, __version_info__))
 
 
 def open(path, create, document_factory=None):
@@ -417,7 +417,7 @@ class DocumentBase(object):
         if json_string is not None:
             try:
                 value = json.loads(json_string)
-            except json.JSONDecodeError:
+            except ValueError:
                 raise InvalidJSON
             if not isinstance(value, dict):
                 raise InvalidJSON
@@ -490,7 +490,7 @@ class DocumentBase(object):
         if json_string is not None:
             try:
                 value = json.loads(json_string)
-            except json.JSONDecodeError:
+            except ValueError:
                 raise InvalidJSON
             if not isinstance(value, dict):
                 raise InvalidJSON
